#include <SDL3/SDL_gpu.h>
#include <pch.h>

SDL_GPUGraphicsPipeline* GraphicsPipelineCreate(SDL_GPUDevice* device, SDL_Window* window,
    SDL_GPUShader* vertex_shader, SDL_GPUShader* fragment_shader) {
    assert(device != NULL);
    assert(window != NULL);
    assert(vertex_shader != NULL);
    assert(fragment_shader != NULL);

    SDL_GPUGraphicsPipeline* pipeline = SDL_CreateGPUGraphicsPipeline(device,
        &(SDL_GPUGraphicsPipelineCreateInfo){
            .target_info = {
		.num_color_targets = 1,
		.color_target_descriptions = (SDL_GPUColorTargetDescription[]){
                    {
			.format = SDL_GetGPUSwapchainTextureFormat(device, window),
		    },
                },
	    },
	    .vertex_input_state = (SDL_GPUVertexInputState){
		.num_vertex_buffers = 1,
		.vertex_buffer_descriptions = (SDL_GPUVertexBufferDescription[]){
                    {
		        .slot = 0,
			.input_rate = SDL_GPU_VERTEXINPUTRATE_VERTEX,
			.instance_step_rate = 0,
			.pitch = sizeof(float) * 3,
		    },
                    {
                        .slot = 1,
                        .input_rate = SDL_GPU_VERTEXINPUTRATE_VERTEX,
                        .instance_step_rate = 0,
                        .pitch = sizeof(float) * 3,
                    }
                },
		.num_vertex_attributes = 2,
		.vertex_attributes = (SDL_GPUVertexAttribute[]){
                    {
		        .buffer_slot = 0,
			.format = SDL_GPU_VERTEXELEMENTFORMAT_FLOAT3,
			.location = 0,
			.offset = 0,
		    },
                    {
                        .buffer_slot = 1,
                        .format = SDL_GPU_VERTEXELEMENTFORMAT_FLOAT3,
                        .location = 1,
                        .offset = 0,
                    }
                }
	    },
	    .primitive_type = SDL_GPU_PRIMITIVETYPE_TRIANGLELIST,
	    .vertex_shader = vertex_shader,
	    .fragment_shader = fragment_shader,
        });
    if(pipeline == NULL) {
        SDL_Log("Failed to create pipeline: %s", SDL_GetError());
    }

    return pipeline;
}
