#include "brimstone/graphics/command_buffer.h"
#include <SDL3/SDL_gpu.h>
#include <pch.h>

bool UploadToBuffer(SDL_GPUDevice* device, const void* data, size_t byte_size, SDL_GPUBuffer* buffer) {
    assert(device != NULL);
    assert(data != NULL);
    assert(buffer != NULL);

    SDL_GPUTransferBuffer* transfer_buffer = SDL_CreateGPUTransferBuffer(device,
        &(SDL_GPUTransferBufferCreateInfo){
            .usage = SDL_GPU_TRANSFERBUFFERUSAGE_UPLOAD,
            .size = byte_size,
        });
    if(transfer_buffer == NULL) {
        SDL_Log("Failed to create transfer buffer: %s", SDL_GetError());
        return false;
    }

    void* mapped_buffer = SDL_MapGPUTransferBuffer(device, transfer_buffer, false);
    if(mapped_buffer == NULL) {
        SDL_Log("Failed to map GPU transfer buffer: %s", SDL_GetError());
        return false;
    }

    memcpy(mapped_buffer, data, byte_size);

    SDL_UnmapGPUTransferBuffer(device, transfer_buffer);

    SDL_GPUCommandBuffer* copy_command_buffer = CommandBufferAcquire(device);
    assert(copy_command_buffer != NULL);

    SDL_GPUCopyPass* copy_pass = CopyPassBegin(copy_command_buffer);
    assert(copy_pass != NULL);

    SDL_UploadToGPUBuffer(copy_pass,
        &(SDL_GPUTransferBufferLocation){
            .transfer_buffer = transfer_buffer,
            .offset = 0,
        },
        &(SDL_GPUBufferRegion){
            .buffer = buffer,
            .offset = 0,
            .size = byte_size,
        },
        false);

    CopyPassEnd(copy_pass);
    bool submit_success = CommandBufferSubmit(copy_command_buffer);
    SDL_ReleaseGPUTransferBuffer(device, transfer_buffer);

    return submit_success;
}
